@@ -24,16 +24,24 @@ from mistral.api.controllers.v2 import resources
 from mistral.api.controllers.v2 import types
 from mistral import context
 from mistral.db.v2 import api as db_api
-from mistral.engine.rpc_backend import rpc
 from mistral import exceptions as exc
+from mistral.rpc import clients as rpc
 from mistral.utils import filter_utils
 from mistral.utils import rest_utils
 from mistral.workflow import states
-from mistral.workflow import utils as wf_utils
+from mistral_lib import actions as ml_actions
 
 
 LOG = logging.getLogger(__name__)
 
+SUPPORTED_TRANSITION_STATES = [
+    states.SUCCESS,
+    states.ERROR,
+    states.CANCELLED,
+    states.PAUSED,
+    states.RUNNING
+]
+
 
 def _load_deferred_output_field(action_ex):
     # We need to refer to this lazy-load field explicitly in
@@ -43,17 +51,20 @@ def _load_deferred_output_field(action_ex):
 
 def _get_action_execution(id):
     with db_api.transaction():
-        action_ex = db_api.get_action_execution(id)
-
-        return _get_action_execution_resource(action_ex)
+        return _get_action_execution_resource(db_api.get_action_execution(id))
 
 
 def _get_action_execution_resource(action_ex):
     _load_deferred_output_field(action_ex)
 
+    return _get_action_execution_resource_for_list(action_ex)
+
+
+def _get_action_execution_resource_for_list(action_ex):
+
     # TODO(nmakhotkin): Get rid of using dicts for constructing resources.
     # TODO(nmakhotkin): Use db_model for this instead.
-    res = resources.ActionExecution.from_dict(action_ex.to_dict())
+    res = resources.ActionExecution.from_db_model(action_ex)
 
     task_name = (action_ex.task_execution.name
                  if action_ex.task_execution else None)
@@ -64,7 +75,7 @@ def _get_action_execution_resource(action_ex):
 
 def _get_action_executions(task_execution_id=None, marker=None, limit=None,
                            sort_keys='created_at', sort_dirs='asc',
-                           fields='', **filters):
+                           fields='', include_output=False, **filters):
     """Return all action executions.
 
     Where project_id is the same as the requester or
@@ -89,12 +100,17 @@ def _get_action_executions(task_execution_id=None, marker=None, limit=None,
     if task_execution_id:
         filters['task_execution_id'] = task_execution_id
 
+    if include_output:
+        resource_function = _get_action_execution_resource
+    else:
+        resource_function = _get_action_execution_resource_for_list
+
     return rest_utils.get_all(
         resources.ActionExecutions,
         resources.ActionExecution,
         db_api.get_action_executions,
         db_api.get_action_execution,
-        resource_function=_get_action_execution_resource,
+        resource_function=resource_function,
         marker=marker,
         limit=limit,
         sort_keys=sort_keys,
@@ -108,7 +124,10 @@ class ActionExecutionsController(rest.RestController):
     @rest_utils.wrap_wsme_controller_exception
     @wsme_pecan.wsexpose(resources.ActionExecution, wtypes.text)
     def get(self, id):
-        """Return the specified action_execution."""
+        """Return the specified action_execution.
+
+        :param id: UUID of action execution to retrieve
+        """
         acl.enforce('action_executions:get', context.ctx())
 
         LOG.info("Fetch action_execution [id=%s]", id)
@@ -119,10 +138,16 @@ class ActionExecutionsController(rest.RestController):
     @wsme_pecan.wsexpose(resources.ActionExecution,
                          body=resources.ActionExecution, status_code=201)
     def post(self, action_ex):
-        """Create new action_execution."""
+        """Create new action_execution.
+
+        :param action_ex: Action to execute
+        """
         acl.enforce('action_executions:create', context.ctx())
 
-        LOG.info("Create action_execution [action_execution=%s]", action_ex)
+        LOG.info(
+            "Create action_execution [action_execution=%s]",
+            action_ex
+        )
 
         name = action_ex.name
         description = action_ex.description or None
@@ -134,14 +159,14 @@ class ActionExecutionsController(rest.RestController):
                 "Please provide at least action name to run action."
             )
 
-        action_ex = rpc.get_engine_client().start_action(
+        values = rpc.get_engine_client().start_action(
             name,
             action_input,
             description=description,
             **params
         )
 
-        return resources.ActionExecution.from_dict(action_ex)
+        return resources.ActionExecution.from_dict(values)
 
     @rest_utils.wrap_wsme_controller_exception
     @wsme_pecan.wsexpose(
@@ -150,33 +175,44 @@ class ActionExecutionsController(rest.RestController):
         body=resources.ActionExecution
     )
     def put(self, id, action_ex):
-        """Update the specified action_execution."""
+        """Update the specified action_execution.
+
+        :param id: UUID of action execution to update
+        :param action_ex: Action execution for update
+        """
         acl.enforce('action_executions:update', context.ctx())
 
         LOG.info(
-            "Update action_execution [id=%s, action_execution=%s]"
-            % (id, action_ex)
+            "Update action_execution [id=%s, action_execution=%s]",
+            id,
+            action_ex
         )
 
-        output = action_ex.output
-
-        if action_ex.state == states.SUCCESS:
-            result = wf_utils.Result(data=output)
-        elif action_ex.state == states.ERROR:
-            if not output:
-                output = 'Unknown error'
-            result = wf_utils.Result(error=output)
-        elif action_ex.state == states.CANCELLED:
-            result = wf_utils.Result(cancel=True)
-        else:
+        if action_ex.state not in SUPPORTED_TRANSITION_STATES:
             raise exc.InvalidResultException(
                 "Error. Expected one of %s, actual: %s" % (
-                    [states.SUCCESS, states.ERROR, states.CANCELLED],
+                    SUPPORTED_TRANSITION_STATES,
                     action_ex.state
                 )
             )
 
-        values = rpc.get_engine_client().on_action_complete(id, result)
+        if states.is_completed(action_ex.state):
+            output = action_ex.output
+
+            if action_ex.state == states.SUCCESS:
+                result = ml_actions.Result(data=output)
+            elif action_ex.state == states.ERROR:
+                if not output:
+                    output = 'Unknown error'
+                result = ml_actions.Result(error=output)
+            elif action_ex.state == states.CANCELLED:
+                result = ml_actions.Result(cancel=True)
+
+            values = rpc.get_engine_client().on_action_complete(id, result)
+
+        if action_ex.state in [states.PAUSED, states.RUNNING]:
+            state = action_ex.state
+            values = rpc.get_engine_client().on_action_update(id, state)
 
         return resources.ActionExecution.from_dict(values)
 
@@ -186,13 +222,13 @@ class ActionExecutionsController(rest.RestController):
                          wtypes.text, wtypes.text, wtypes.text,
                          wtypes.text, wtypes.text, wtypes.text, types.uuid,
                          wtypes.text, wtypes.text, bool, types.jsontype,
-                         types.jsontype, types.jsontype, wtypes.text)
+                         types.jsontype, types.jsontype, wtypes.text, bool)
     def get_all(self, marker=None, limit=None, sort_keys='created_at',
                 sort_dirs='asc', fields='', created_at=None, name=None,
                 tags=None, updated_at=None, workflow_name=None,
                 task_name=None, task_execution_id=None, state=None,
                 state_info=None, accepted=None, input=None, output=None,
-                params=None, description=None):
+                params=None, description=None, include_output=False):
         """Return all tasks within the execution.
 
         Where project_id is the same as the requester or
@@ -234,6 +270,8 @@ class ActionExecutionsController(rest.RestController):
                            time and date.
         :param updated_at: Optional. Keep only resources with specific latest
                            update time and date.
+        :param include_output: Optional. Include the output for all executions
+                               in the list
         """
         acl.enforce('action_executions:list', context.ctx())
 
@@ -254,9 +292,15 @@ class ActionExecutionsController(rest.RestController):
             description=description
         )
 
-        LOG.info("Fetch action_executions. marker=%s, limit=%s, "
-                 "sort_keys=%s, sort_dirs=%s, filters=%s",
-                 marker, limit, sort_keys, sort_dirs, filters)
+        LOG.info(
+            "Fetch action_executions. marker=%s, limit=%s, "
+            "sort_keys=%s, sort_dirs=%s, filters=%s",
+            marker,
+            limit,
+            sort_keys,
+            sort_dirs,
+            filters
+        )
 
         return _get_action_executions(
             marker=marker,
@@ -264,13 +308,17 @@ class ActionExecutionsController(rest.RestController):
             sort_keys=sort_keys,
             sort_dirs=sort_dirs,
             fields=fields,
+            include_output=include_output,
             **filters
         )
 
     @rest_utils.wrap_wsme_controller_exception
     @wsme_pecan.wsexpose(None, wtypes.text, status_code=204)
     def delete(self, id):
-        """Delete the specified action_execution."""
+        """Delete the specified action_execution.
+
+        :param id: UUID of action execution to delete
+        """
         acl.enforce('action_executions:delete', context.ctx())
 
         LOG.info("Delete action_execution [id=%s]", id)
@@ -302,13 +350,14 @@ class TasksActionExecutionController(rest.RestController):
                          wtypes.text, types.uniquelist, wtypes.text,
                          wtypes.text, wtypes.text, wtypes.text, wtypes.text,
                          wtypes.text, bool, types.jsontype, types.jsontype,
-                         types.jsontype, wtypes.text)
+                         types.jsontype, wtypes.text, bool)
     def get_all(self, task_execution_id, marker=None, limit=None,
                 sort_keys='created_at', sort_dirs='asc', fields='',
                 created_at=None, name=None, tags=None,
                 updated_at=None, workflow_name=None, task_name=None,
                 state=None, state_info=None, accepted=None, input=None,
-                output=None, params=None, description=None):
+                output=None, params=None, description=None,
+                include_output=None):
         """Return all tasks within the execution.
 
         Where project_id is the same as the requester or
@@ -350,6 +399,8 @@ class TasksActionExecutionController(rest.RestController):
                            time and date.
         :param updated_at: Optional. Keep only resources with specific latest
                            update time and date.
+        :param include_output: Optional. Include the output for all executions
+                               in the list
         """
         acl.enforce('action_executions:list', context.ctx())
 
@@ -370,9 +421,15 @@ class TasksActionExecutionController(rest.RestController):
             description=description
         )
 
-        LOG.info("Fetch action_executions. marker=%s, limit=%s, "
-                 "sort_keys=%s, sort_dirs=%s, filters=%s",
-                 marker, limit, sort_keys, sort_dirs, filters)
+        LOG.info(
+            "Fetch action_executions. marker=%s, limit=%s, "
+            "sort_keys=%s, sort_dirs=%s, filters=%s",
+            marker,
+            limit,
+            sort_keys,
+            sort_dirs,
+            filters
+        )
 
         return _get_action_executions(
             marker=marker,
@@ -380,13 +437,18 @@ class TasksActionExecutionController(rest.RestController):
             sort_keys=sort_keys,
             sort_dirs=sort_dirs,
             fields=fields,
+            include_output=include_output,
             **filters
         )
 
     @rest_utils.wrap_wsme_controller_exception
     @wsme_pecan.wsexpose(resources.ActionExecution, wtypes.text, wtypes.text)
     def get(self, task_execution_id, action_ex_id):
-        """Return the specified action_execution."""
+        """Return the specified action_execution.
+
+        :param task_execution_id: Task execution UUID
+        :param action_ex_id: Action execution UUID
+        """
         acl.enforce('action_executions:get', context.ctx())
 
         LOG.info("Fetch action_execution [id=%s]", action_ex_id)
